@@ -1,5 +1,5 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2013,2014,2015,2016
+;;  Copyright (C) 2013,2014,2015,2016,2017,2018
 ;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
@@ -19,6 +19,7 @@
 
 (define-module (artanis tpl parser)
   #:use-module (artanis utils)
+  #:use-module (artanis env)
   #:use-module (artanis tpl utils)
   #:use-module (artanis tpl lexer)
   #:use-module (system base lalr)
@@ -30,18 +31,22 @@
 (define (gen-command cmd args)
   (case cmd
     ((include)
-     (if (file-exists? args)
-         (apply cat args #f)
-         (throw 'artanis-err 500 "Included file in template doesn't exist!" args)))
+     (let ((filename (string-trim-both
+                      (format #f "~a/pub/~a" (current-toplevel) args))))
+       (if (file-exists? filename)
+           (format #f "~s" (cat filename #f))
+           (throw 'artanis-err 500 gen-command
+                  "Included file `~a' in template doesn't exist!" filename))))
     ((css)
-     (format #f "<link rel=\"stylesheet\" href=\"/pub/css/~a\">" args))
+     (format #f "\"<link rel=\\\"stylesheet\\\" href=\\\"/pub/css/~a\\\">\"" args))
     ((icon)
-     (format #f "<link rel=\"icon\" href=\"/pub/img/~a\" type=\"image/x-icon\">" args))
+     (format #f "\"<link rel=\\\"icon\\\" href=\\\"/pub/img/~a\\\" type=\\\"image/x-icon\\\">\"" args))
     ((js)
-     (format #f "<script type=\"text/javascript\" src=\"/pub/js/~a\"> </script>" args))
+     (format #f "\"<script type=\\\"text/javascript\\\" src=\\\"/pub/js/~a\\\"> </script>\"" args))
     (else
-     (throw 'artanis-err 500 "Invalid command in template!" cmd))))
-     
+     (throw 'artanis-err 500 gen-command
+            "Invalid command `~a' in template!" cmd))))
+
 (define (make-parser)
   (lalr-parser
    (code disp-code html command) ; terminal tokens
@@ -51,7 +56,7 @@
          (*eoi*) : *eof-object*)
 
    (tpl (html) : (string-trim-both $1)
-        (command) : (gen-command (car $1) (cadr $1))
+        (command) : (string-concatenate `("(display " ,(gen-command (car $1) (cdr $1)) ")"))
         (program) : $1)
 
    (program (code) : $1
