@@ -1,5 +1,5 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2013,2014,2015
+;;  Copyright (C) 2013,2014,2015,2016,2017
 ;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
@@ -37,15 +37,23 @@
   #:use-module (ice-9 local-eval)
   #:use-module (ice-9 receive)
   #:use-module (ice-9 q)
+  #:use-module (ice-9 control)
   #:use-module (web http)
   #:use-module (web request)
+  #:use-module (web uri)
   #:use-module ((rnrs)
                 #:select (get-bytevector-all utf8->string put-bytevector
-                          bytevector-u8-ref string->utf8 bytevector-length))
-  #:export (regexp-split hash-keys cat bv-cat get-global-time
-            get-local-time string->md5 unsafe-random string-substitute
-            get-file-ext get-global-date get-local-date uri-decode
-            nfx static-filename remote-info seconds-now local-time-stamp
+                          bytevector-u8-ref string->utf8 bytevector-length
+                          make-bytevector bytevector-s32-native-ref bytevector?
+                          define-record-type record-rtd record-accessor
+                          get-string-all))
+  #:export (regexp-split hash-keys cat bv-cat get-global-time sanitize-response
+            build-response write-response get-local-time string->md5 unsafe-random
+            uri-encode uri-decode response-version response-code response-connection
+            request-headers response-port write-response-body read-request request-uri
+            request-method request-content-length request-port read-request-body
+            response-content-length get-file-ext get-global-date get-local-date
+            string-substitute nfx static-filename remote-info seconds-now local-time-stamp
             parse-date write-date make-expires export-all-from-module!
             alist->hashtable expires->time-utc local-eval-string
             time-expired? valid-method? mmap munmap get-random-from-dev
@@ -62,14 +70,25 @@
             sxml->xml-string run-after-request! run-before-response!
             make-pipeline HTML-entities-replace eliminate-evil-HTML-entities
             generate-kv-from-post-qstr handle-proper-owner
-            generate-data-url verify-ENTRY
+            generate-data-url verify-ENTRY exclude-dbd
             draw-expander remove-ext scan-app-components cache-this-route!
             dump-route-from-cache generate-modify-time delete-directory
-            handle-existing-file check-drawing-method
+            handle-existing-file check-drawing-method DEBUG
             subbv->string subbv=? bv-read-line bv-read-delimited put-bv
             bv-u8-index bv-u8-index-right build-bv-lookup-table filesize
             plist-remove gen-migrate-module-name try-to-load-migrate-cache
-            flush-to-migration-cache gen-local-conf-file with-dbd)
+            flush-to-migration-cache gen-local-conf-file with-dbd
+            call-with-sigint define-box-type make-box-type unbox-type
+            ::define did-not-specify-parameter colorize-string-helper
+            colorize-string WARN-TEXT ERROR-TEXT REASON-TEXT
+            NOTIFY-TEXT STATUS-TEXT get-trigger get-family get-addr request-path
+            response-keep-alive? request-keep-alive?
+            procedure-name->string proper-toplevel gen-content-length
+            make-file-sender file-sender? file-sender-size file-sender-thunk
+            get-string-all-with-detected-charset make-unstop-exception-handler
+            artanis-log exception-from-client exception-from-server render-sys-page
+            bv-copy/share bv-backward artanis-list-matches get-syspage
+            artanis-sys-response)
   #:re-export (the-environment))
 
 ;; There's a famous rumor that 'urandom' is safer, so we pick it.
@@ -83,8 +102,24 @@
             str)))))
 
 (define uri-decode (@ (web uri) uri-decode))
+(define uri-encode (@ (web uri) uri-encode))
 (define parse-date (@@ (web http) parse-date))
 (define write-date (@@ (web http) write-date))
+(define build-response (@ (web response) build-response))
+(define write-response (@ (web response) write-response))
+(define response-version (@ (web response) response-version))
+(define response-code (@ (web response) response-code))
+(define response-connection (@ (web response) response-connection))
+(define response-port (@ (web response) response-port))
+(define write-response-body (@ (web response) write-response-body))
+(define response-content-length (@ (web response) response-content-length))
+(define read-request (@ (web request) read-request))
+(define read-request-body (@ (web request) read-request-body))
+(define request-uri (@ (web request) request-uri))
+(define request-headers (@ (web request) request-headers))
+(define request-method (@ (web request) request-method))
+(define request-content-length (@ (web request) request-content-length))
+(define request-port (@ (web request) request-port))
 
 (define-syntax-rule (local-eval-string str e)
   (local-eval 
@@ -99,10 +134,10 @@
     ht))
 
 (eval-when (eval load compile)
-           (define (export-all-from-module! module-name)
-             (let ((mod (resolve-module module-name)))
-               (module-for-each (lambda (s m) 
-                                  (module-add! (current-module) s m)) mod))))
+  (define (export-all-from-module! module-name)
+    (let ((mod (resolve-module module-name)))
+      (module-for-each (lambda (s m)
+                         (module-add! (current-module) s m)) mod))))
 
 (define (time-expired? expires)
   (if expires
@@ -172,7 +207,7 @@
         (display str out)
         str)))
 
-;; WARN: besure that you've already checked the file exists before!!!
+;; WARN: besure that you've already checked the file existtance before!!!
 (define* (bv-cat file/port #:optional (out (current-output-port)))
   (define get-bytevector-all (@ (rnrs io ports) get-bytevector-all))
   (let ((bv (if (port? file/port)
@@ -222,7 +257,9 @@
         (lp (cddr rest) e #f))))))
 
 (define-syntax-rule (static-filename path)
-  (substring/shared path 1))
+  (if (current-toplevel)
+      (format #f "~a/~a" (current-toplevel) path)
+      (format #f "./~a" path)))
 
 (define-syntax-rule (request-ip req)
   ;; TODO: support AF_INET6 in the future
@@ -243,7 +280,7 @@
 (define (valid-method? method)
   (if (and (member method *methods-list*) (allowed-method? method))
       method
-      (throw 'artanis-err 405 "invalid HTTP method" method)))
+      (throw 'artanis-err 405 valid-method? "invalid HTTP method `~a'" method)))
 
 ;; -------------- mmap ---------------------
 (define-public ACCESS_COPY              #x3)
@@ -274,16 +311,21 @@
 (define %mmap
   (pointer->procedure '*
                       (dynamic-func "mmap" *libc-ffi*)
-                      (list '* size_t int int int size_t)))
+                      (list '* size_t int int int size_t) #:return-errno? #t))
 (define %munmap
   (pointer->procedure int
                       (dynamic-func "munmap" *libc-ffi*)
-                      (list '* size_t)))
+                      (list '* size_t) #:return-errno? #t))
 (define* (mmap size #:key (addr %null-pointer) (fd -1) (prot MAP_SHARED) 
-               (flags PROT_READ) (offset 0))
-  (pointer->bytevector (%mmap addr size prot flags fd offset) size))
-(define (munmap bv size)
-  (%munmap (bytevector->pointer bv size) size))
+               (flags PROT_READ) (offset 0) (bv? #f))
+  (let ((ret (if bv? (lambda (p) (pointer->bytevector p size)) identity)))
+    (ret (%mmap addr size prot flags fd offset))))
+(define (munmap bv/pointer size)
+  (cond
+   ((bytevector? bv/pointer)
+    (%munmap (bytevector->pointer bv/pointer size) size))
+   ((pointer? bv/pointer)
+    (%munmap bv/pointer size))))
 
 ;; FIXME: what if len is not even?
 (define (string->byteslist str step base)
@@ -418,7 +460,8 @@
       (case mode
         ((normal) str)
         ((db) (string-concatenate (list "\"" (->string str) "\"")))
-        (else (throw 'artanis-err 500 "%make-string-template: invalid mode" mode)))))
+        (else (throw 'artanis-err 500 %make-string-template
+                     "invalid mode `~a'" mode)))))
   (define (optimize rev-items tail)
     (cond ((null? rev-items) tail)
           ((not (string? (car rev-items)))
@@ -453,8 +496,7 @@
             item
             (or (and=> (get-the-val keyword-args (car item)) ->string)
                 (cdr item) ; default value
-                (throw 'artanis-err 500
-                       "(utils)item->string: Missing keyword" (car item)))))
+                "")))
       (string-concatenate (map item->string items)))))
 
 ;; the normal mode, no double quotes for vals
@@ -589,7 +631,8 @@
    ((string? col) (string->symbol (-> c string-downcase)))
    ((symbol? col) (-> col symbol-downcase))
    ((keyword? col) (normalize-column (keyword->string col) ci?))
-   (else (throw 'artanis-err 500 "normalize-column: Invalid type of column" col))))
+   (else (throw 'artanis-err 500 normalize-column
+                "Invalid type of column `~a'" col))))
 
 (define* (sxml->xml-string sxml #:key (escape? #f))
   (call-with-output-string
@@ -651,8 +694,9 @@
   (define cook (if no-evil? eliminate-evil-HTML-entities identity))
   (define (%convert lst)
     (match lst
-      ((k v) (list (key-converter k) v))
-      (else (throw 'artanis-err 500 "generate-kv-from-post-qstr: Fatal! Can't be here!" lst))))
+      ((k v) (list (key-converter k) ((current-encoder) v)))
+      (else (throw 'artanis-err 500 generate-kv-from-post-qstr
+                   "Fatal! Can't be here!" lst))))
   (define (-> x)
     (string-trim-both x (lambda (c) (member c '(#\sp #\: #\return)))))
   (map (lambda (x)
@@ -680,7 +724,7 @@
                  "Maybe you run Artanis as unprivileged user? (say, not as root)\n"
                  (current-error-port)))
                ('(system-error . "No such file or directory")
-                (throw 'artanis-err 500 (->err-reason exe reason) file))
+                (throw 'artanis-err 500 handle-proper-owner (->err-reason exe reason) file))
                (else (apply throw k e)))))))
 
 ;; According to wiki, here's the standard format of data_url_scheme:
@@ -701,19 +745,21 @@
       ((or (? symbol?) (? string?))
        (or (and (get-conf 'debug-mode) (mime-check mime) (format #f "~a" mime))
            (format #f "~a" mime)))
-      (else (throw 'artanis-err 500
-                   "generate-data-url: Invalid MIME! Should be symbol or string"
+      (else (throw 'artanis-err 500 generate-data-url
+                   "Invalid MIME! Should be symbol or string"
                    mime))))
   (let ((b64 (base64-encode bv/str)))
     (string-concatenate (list "data:" (->mime) (->crypto) (->charset) "," b64))))
 
 (define (verify-ENTRY entry)
+  (format #t "entry: ~a~%" entry)
   (cond
-   ((not (file-exists? entry)) #f)
+   ((not (file-exists? entry))
+    (format (artanis-current-output) "ENTRY file is missing!~%")
+    #f)
    (else
-    (let* ((line (call-with-input-file entry read-line))
-           (m (string-match "Artanis top-level: (.*)" line)))
-      (and m (string=? (match:substring m 1) (dirname entry)))))))
+    (let ((line (call-with-input-file entry read-line)))
+      (string=? line ";; This an Artanis ENTRY file, don't remove it!")))))
 
 (define-syntax draw-expander
   (syntax-rules (rule options method)
@@ -735,7 +781,8 @@
          (scandir (format #f "~a/app/~a/" toplevel component)
                   (lambda (f)
                     (not (or (string=? f ".")
-                             (string=? f ".."))))))))
+                             (string=? f "..")
+                             (string=? f ".gitkeep"))))))))
 
 (define (cache-this-route! url meta)
   (define (write-header port)
@@ -908,13 +955,13 @@
           (lp (cddr next) (list (car next) (cadr next)) ret)))
      (else (lp (cdr next) kk (cons (car next) ret))))))
 
-(define *name-re* (string->sre "([^.]+)\\.scm"))
+(define *name-re* (string->irregex "([^.]+)\\.scm"))
 (define (gen-migrate-module-name f)
   (cond
    ((irregex-search *name-re* (basename f))
     => (lambda (m) (irregex-match-substring m 1)))
-   (else (throw 'artanis-err 500
-                "Migrate: wrong parsing of module name, shouldn't be here!" f))))
+   (else (throw 'artanis-err 500 gen-migrate-module-name
+                "Wrong parsing of module name, shouldn't be here!" f))))
 
 (define (try-to-load-migrate-cache name)
   (let ((file (format #f "~a/tmp/cache/migration/~a.scm" (current-toplevel) name)))
@@ -945,7 +992,406 @@
     (cond
      ((or (and (list? dbd0) (memq dbd1 dbd0)) (eq? dbd1 dbd0)) body ...)
      (else
-      (throw 'artanis-err 500
-             (format #f "This is only supported by `~a', but the current dbd is `~a'"
-                     dbd0 dbd1)
+      (throw 'artanis-err 500 'with-dbd
+             "This is only supported by `~a', but the current dbd is `~a'"
+             dbd0 dbd1
              'body ...)))))
+
+(define-syntax-rule (exclude-dbd dbds body ...)
+  (let ((dbd (get-conf '(db dbd))))
+    (cond
+     ((memq dbd dbds)
+      (throw 'artanis-err 500 'exclude-dbd
+             "This isn't supported by `~a', please check it out again!"
+             dbds
+             'body ...))
+     (else body ...))))
+
+(define-syntax-rule (DEBUG fmt args ...)
+  (when (get-conf 'debug-mode)
+        (format (artanis-current-output) fmt args ...)))
+
+(define call-with-sigint
+  (if (not (provided? 'posix))
+      (lambda (thunk handler-thunk) (thunk))
+      (lambda (thunk handler-thunk)
+        (let ((handler #f))
+          (catch 'interrupt
+            (lambda ()
+              (dynamic-wind
+                (lambda ()
+                  (set! handler
+                        (sigaction SIGINT (lambda (sig) (throw 'interrupt)))))
+                thunk
+                (lambda ()
+                  (if handler
+                      ;; restore Scheme handler, SIG_IGN or SIG_DFL.
+                      (sigaction SIGINT (car handler) (cdr handler))
+                      ;; restore original C handler.
+                      (sigaction SIGINT #f)))))
+            (lambda (k . _) (handler-thunk)))))))
+
+(define-syntax-rule (define-box-type name)
+  (define-record-type name (fields treasure)))
+
+(define-macro (make-box-type bt v)
+  (list (symbol-append 'make- bt) v))
+
+(define-syntax-rule (box-type-treasure t)
+  (record-accessor (record-rtd t) 0))
+
+(define-syntax-rule (unbox-type t)
+  (let ((treasure-getter (box-type-treasure t)))
+    (treasure-getter t)))
+
+(define (socket-port? sp)
+  (and (port? sp) (eq? (port-filename sp) 'socket)))
+
+(define (detect-type-name o)
+  (define r6rs-record? (@ (rnrs) record?))
+  (define r6rs-record-type-name (@ (rnrs) record-type-name))
+  (define guile-specific-record? (@ (guile) record?))
+  (define (guile-specific-record-name o)
+    ((@ (guile) record-type-name)
+     ((@ (guile) record-type-descriptor) o)))
+  (cond
+   ;; NOTE: record? is for R6RS record-type, but record-type? is not
+   ((r6rs-record? o) (r6rs-record-type-name (record-rtd o)))
+   ((guile-specific-record? o) (guile-specific-record-name o))
+   ((symbol? o) 'symbol)
+   ((string? o) 'string)
+   ((integer? o) (if (positive? o) '+int '-int))
+   ((number? o) 'num)
+   ((thunk? o) 'thunk)
+   ((procedure? o) 'proc)
+   ((vector? o) 'vector)
+   ((pair? o) 'pair)
+   ((list? o) 'list)
+   ((bytevector? o) 'bv)
+   ((port? o) 'port)
+   ((boolean? o) 'boolean)
+   (else 'ANY)))
+
+(define (check-args-types op args)
+  (define (check-eq? actual-type expect-type)
+    (case expect-type
+      ((+int) (eq? actual-type '+int))
+      ((-int) (eq? actual-type '-int))
+      ((int) (memq actual-type '(-int +int)))
+      (else (eq? expect-type actual-type))))
+  (match (procedure-property op 'type-anno)
+    (((targs ...) '-> (func-types ...))
+     (for-each
+      (lambda (v e)
+        (or (eq? e 'ANY)
+            (check-eq? (detect-type-name v) e)
+            (begin
+              (DEBUG "(~{~a~^ ~}) =? (~{~a~^ ~})~%" targs args)
+              (throw 'artanis-err 500 check-args-types
+                     "~a: Argument ~a is a `~a' type, but I expect type `~a'"
+                     op v (detect-type-name v) e))))
+      args targs))
+    (else (throw 'artanis-err 500 check-args-types "Invalid type annotation `~a'"
+                 (procedure-property op 'type-anno)))))
+
+(define (check-function-types op fret)
+  (match (procedure-property op 'type-anno)
+    (((targs ...) '-> (func-types ...))
+     (for-each
+      (lambda (v e)
+        (or (eq? e 'ANY)
+            (eq? (detect-type-name v) e)
+            (throw 'artanis-err 500 check-function-types
+                   "`Return value ~a(~a) is expected to be type `~a'"
+                   v (detect-type-name v) e)))
+      fret func-types))
+    (else (throw 'artanis-err 500 check-function-types
+                 "Invalid type annotation `~a'"
+                 (procedure-property op 'type-anno)))))
+
+(define (detect-and-set-type-anno! op ftypes atypes)
+  (let ((type `(,atypes -> ,ftypes)))
+    (set-procedure-property! op 'type-anno type)
+    type))
+
+;; NOTE: This macro can detect multi return values.
+;; TODO:
+;; 1. support multi-types, say, string/bv, maybe not easy to do it faster?
+(define-syntax ::define
+  (syntax-rules (-> :anno:)
+    ((_ (op args ...) (:anno: (targs ...) -> func-types ...) body ...)
+     (begin
+       (define (op args ...)
+         (when (get-conf 'debug-mode) (check-args-types op (list args ...)))
+         (call-with-values
+             (lambda () body ...)
+           (lambda ret
+             (when (get-conf 'debug-mode)
+                   (eq? (detect-type-name ret) (check-function-types op ret)))
+             (apply values ret))))
+       (detect-and-set-type-anno! op '(func-types ...) '(targs ...))))))
+
+(define-syntax-rule (did-not-specify-parameter what)
+  (format #f "`current-~a' isn't specified, it's likely a bug!" what))
+
+;; Text-coloring helper functions, borrowed from guile-colorized
+(define *color-list*
+  `((CLEAR       .   "0")
+    (RESET       .   "0")
+    (BOLD        .   "1")
+    (DARK        .   "2")
+    (UNDERLINE   .   "4")
+    (UNDERSCORE  .   "4")
+    (BLINK       .   "5")
+    (REVERSE     .   "6")
+    (CONCEALED   .   "8")
+    (BLACK       .  "30")
+    (RED         .  "31")
+    (GREEN       .  "32")
+    (YELLOW      .  "33")
+    (BLUE        .  "34")
+    (MAGENTA     .  "35")
+    (CYAN        .  "36")
+    (WHITE       .  "37")
+    (ON-BLACK    .  "40")
+    (ON-RED      .  "41")
+    (ON-GREEN    .  "42")
+    (ON-YELLOW   .  "43")
+    (ON-BLUE     .  "44")
+    (ON-MAGENTA  .  "45")
+    (ON-CYAN     .  "46")
+    (ON-WHITE    .  "47")))
+
+(define (get-color color)
+  (assq-ref *color-list* color))
+
+(define (generate-color colors)
+  (let ((color-list
+         (filter-map get-color colors)))
+    (if (null? color-list)
+        ""
+        (string-append "\x1b[" (string-join color-list ";" 'infix) "m"))))
+
+(define* (colorize-string-helper color str control #:optional (rl-ignore #f))
+  (if rl-ignore
+      (string-append "\x01" (generate-color color) "\x02" str "\x01" (generate-color control) "\x02")
+      (string-append (generate-color color) str (generate-color control))))
+
+(define* (colorize-string str color)
+  "Example: (colorize-string \"hello\" '(BLUE BOLD))"
+  (colorize-string-helper color str '(RESET) (using-readline?)))
+
+(define-syntax-rule (WARN-TEXT str) (colorize-string str '(YELLOW)))
+(define-syntax-rule (ERROR-TEXT str) (colorize-string str '(RED)))
+(define-syntax-rule (REASON-TEXT str) (colorize-string str '(CYAN)))
+(define-syntax-rule (NOTIFY-TEXT str) (colorize-string str '(WHITE)))
+(define-syntax-rule (STATUS-TEXT num) (colorize-string (object->string num)'(WHITE)))
+
+(define (get-trigger)
+  (case (get-conf '(server trigger))
+    ((edge) (@ (artanis server epoll) EPOLLET))
+    ((level) 0)
+    (else (throw 'artanis-err 500 get-trigger "Invalid (server trigger)!"
+                 (get-conf '(server trigger))))))
+
+(define (get-family)
+  (case (get-conf '(host family))
+    ((ipv4) AF_INET)
+    ((ipv6) AF_INET6)
+    (else (throw 'artanis-err 500 get-family "Invalid (host family)!"
+                 (get-conf '(host family))))))
+        
+(define (get-addr)
+  (let ((host (get-conf '(host addr)))
+        (family (get-family)))
+    (inet-pton family host)))
+
+(define (request-path req)
+  (uri-path (request-uri req)))
+
+(define (response-keep-alive? response)
+  (let ((v (response-version response)))
+    (and (or (< (response-code response) 400)
+             (= (response-code response) 404))
+         (case (car v)
+           ((1)
+            (case (cdr v)
+              ;; NOTE: HTTP/1.1 treat all connection keep-alive
+              ;;       unless it requires `close' explicityly
+              ((1) (not (memq 'close (response-connection response))))
+              ;; HTTP/1.0 needs explicit keep-alive notice
+              ((0) (memq 'keep-alive (response-connection response)))))
+           (else #f)))))
+
+;; NOTE: The order matters
+(define (request-keep-alive? request)
+  (or (equal? (request-upgrade request) '(websocket))
+      (equal? (request-connection request) '(keep-alive))
+      (equal? (request-version request) '(1 . 1))))
+
+(define (procedure-name->string proc)
+  (symbol->string (procedure-name proc)))
+
+(define-syntax-rule (proper-toplevel)
+  (or (current-toplevel) ""))
+
+(define-record-type file-sender
+  (fields size thunk))
+
+(define (gen-content-length body)
+  (let ((get-length (lambda ()
+                      (cond
+                       ((bytevector? body) (bytevector-length body))
+                       ((file-sender? body) (file-sender-size body))
+                       ((string? body)
+                        (throw 'artanis-err 500 gen-content-length
+                               "BUG: body should have been converted to bytevector! ~a"
+                               body))
+                       (else (throw 'artanis-err 500 gen-content-length
+                                    "Invalid body ~a" body))))))
+    `(content-length . ,(if body (get-length) 0))))
+
+(define (get-string-all-with-detected-charset filename)
+  (call-with-input-file filename
+    (lambda (port)
+      (set-port-encoding! port (get-conf '(server charset)))
+      (get-string-all port))))
+
+(define (get-syspage file)
+  (let ((local-syspage (format #f "~a/sys/pages/~a"
+                               (current-toplevel) file)))
+    (if (file-exists? local-syspage)
+        local-syspage
+        (format #f "~a/~a" (get-conf '(server syspage path)) file))))
+
+(define (syspage-show file)
+  (bv-cat (get-syspage file) #f))
+
+;; ENHANCE: use colored output
+(define* (artanis-log blame-who? status mime #:key (port (current-error-port))
+                      (request #f))
+  (case blame-who?
+    ((client)
+     (when (not request)
+       (error "artanis-log: Fatal bug! Request shouldn't be #f here!~%"))
+     (let* ((uri (request-uri request))
+            (path (uri-path uri))
+            (qstr (uri-query uri))
+            (method (request-method request)))
+       (format port "[Remote] ~a @ ~a~%" (remote-info request) (local-time-stamp))
+       (format port "[Request] method: ~a, path: ~a, query: ~a~%" method path qstr)
+       (format port "[Response] status: ~a, MIME: ~a~%~%" status mime)))
+    ((server)
+     (format port "[Server] ~a @ ~a~%" (get-conf '(host addr)) (local-time-stamp))
+     (format port "[Response] status: ~a, MIME: ~a~%~%" status mime))
+    (else (error "artanis-log: Fatal BUG here!"))))
+
+(define (render-sys-page blame-who? status request)
+  (define-syntax-rule (status->page s)
+    (format #f "~a.html" s))
+  (artanis-log blame-who? status 'text/html #:request request)
+  (let* ((charset (get-conf '(server charset)))
+         (mtime (generate-modify-time (current-time)))
+         (response
+          (build-response #:code status
+                          #:headers `((server . ,(get-conf '(server info)))
+                                      (last-modified . ,mtime)
+                                      (content-type . (text/html (charset . ,charset))))))
+         (body (syspage-show (status->page status))))
+    (if (eq? (get-conf '(server engine)) 'guile)
+        (values response body)
+        (values response body 'exception))))
+
+(define (format-status-page/client status request)
+  (format (current-error-port) (ERROR-TEXT "[EXCEPTION] ~a is abnormal request, status: ~a, ")
+          (uri-path (request-uri request)) status)
+  (display "rendering a sys page for it...\n" (current-error-port)) 
+  (render-sys-page 'client status request))
+
+(define (format-status-page/server status)
+  (format (current-error-port) "[SERVER ERROR] Internal error from server-side, ")
+  (format (current-error-port) "rendering a ~a page for client ...~%" status)
+  (render-sys-page 'server status #f))
+
+(define (exception-from-client request)
+  (lambda (status)
+    (format-status-page/client status request)))
+
+(define (exception-from-server)
+  (lambda (status)
+    (format-status-page/server status)))
+
+(define *rf-re* (string->irregex ".*/artanis/artanis/(.*)$"))
+(define (->reasonable-file filename)
+  (if (string? filename)
+      (let ((m (irregex-search *rf-re* filename)))
+        (if m
+            (format #f "artanis/~a" (irregex-match-substring m 1))
+            filename))
+          "In unknown file"))
+(define-syntax-rule (make-unstop-exception-handler syspage-generator)
+  (let ((port (current-error-port))
+        (filename (current-filename)))
+    (lambda (k . e)
+      (match e
+        (((? procedure? subr) (? string? msg) . args)
+         (format port "Captured in <~a>~%" (WARN-TEXT (->reasonable-file filename)))
+         (when subr (format port "In procedure ~a :~%"
+                            (WARN-TEXT (procedure-name->string subr))))
+         (apply format port
+                (REASON-TEXT (string-append "[REASON] " msg))
+                args)
+         (newline port))
+        (((? integer? status) (or (? symbol? subr) (? procedure? subr))
+          (? string? msg) . args)
+         (format port "HTTP ~a~%" (STATUS-TEXT status))
+         (format port "Captured in <~a>~%" (WARN-TEXT (->reasonable-file filename)))
+         (when subr (format port "Threw in procedure ~a :~%"
+                            (WARN-TEXT (if (procedure? subr)
+                                           (procedure-name->string subr)
+                                           subr))))
+         (apply format port
+                (REASON-TEXT (string-append "[REASON] " msg))
+                args)
+         (newline port)
+         (syspage-generator status))
+        (else
+         (format port "~a - ~a~%"
+                 (WARN-TEXT
+                  "BUG: invalid exception format, but we throw it anyway!")
+                 e)
+         (apply throw k e))))))
+
+(define* (bv-copy/share bv #:key (from 0) (type 'vu8)
+                        (size (- (bytevector-length bv) from)))
+  (when (> size (- (bytevector-length bv) from))
+    (error bv-copy/share
+           (format #f "Size(~a) is larger than the length(~a) - from(~a)!"
+                   size (bytevector-length bv) from)))
+  (when (>= from (bytevector-length bv))
+    (error bv-copy/share
+           (format #f "Can't copy from the end of the bytevector (~a)!"
+                   from)))
+  (let* ((ptr (bytevector->pointer bv))
+         (new-ptr (make-pointer (+ (pointer-address ptr) from))))
+    (pointer->bytevector new-ptr size 0 type)))
+
+(define* (bv-backward bv offset #:key (type 'vu8) (extend 0))
+  (let* ((ptr (bytevector->pointer bv))
+         (new-ptr (make-pointer (- (pointer-address ptr) offset)))
+         (len (bytevector-length bv)))
+    (pointer->bytevector new-ptr (+ len extend) 0 type)))
+
+(define (artanis-list-matches irx str)
+  (let lp ((start 0) (ret '()))
+    (let ((m (irregex-search irx str start)))
+      (if m
+          (lp (irregex-match-end-index m) (cons m ret))
+          ret))))
+
+(define (artanis-sys-response status port bv-body)
+  (build-response #:code status
+                  #:port port
+                  #:headers `((server . ,(get-conf '(server info)))
+                              ,(gen-content-length bv-body)
+                              (content-type . (text/html)))))
